@@ -72,6 +72,9 @@ namespace OrthancWSI
     bool          forceOpenSlide_;
     unsigned int  padding_;
 
+    // New in release 3.3
+    Orthanc::Encoding  encoding_;
+
   public:
     DicomizerParameters();
 
@@ -300,5 +303,15 @@ namespace OrthancWSI
     {
       return forceOpenSlide_;
     }
+
+    Orthanc::Encoding GetEncoding() const
+    {
+      return encoding_;
+    }
+
+    void SetEncoding(Orthanc::Encoding encoding)
+    {
+      encoding_ = encoding;
+    }
   };
 }
