@@ -23,6 +23,8 @@
 
 #pragma once
 
+#include <string>
+
 namespace OrthancWSI
 {
   class ImagedVolumeParameters
@@ -84,6 +86,8 @@ namespace OrthancWSI
       offsetY_ = offset;
     }
 
+    std::string GetImageOrientationSlide() const;
+
     void GetLocation(float& physicalX,
                      float& physicalY,
                      unsigned int imageX,
