@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -24,6 +24,7 @@
 #include "../PrecompiledHeadersWSI.h"
 #include "DicomPyramid.h"
 
+#include "../ImageToolbox.h"
 #include "../DicomToolbox.h"
 
 #include <Compatibility.h>
@@ -98,10 +99,15 @@ namespace OrthancWSI
         std::vector<std::string> tokens;
         Orthanc::Toolbox::TokenizeString(tokens, instance->GetImageType(), '\\');
 
-        // Don't consider the thumbnail and overview as part of the DICOM pyramid (new in 1.0)
-        if (tokens.size() < 2 ||
-            (tokens[1] != "THUMBNAIL" &&
-             tokens[1] != "OVERVIEW"))
+        /**
+         * Don't consider the "LABEL" and "OVERVIEW" that are not part
+         * of the pyramid. Originally introduced in release 1.0, but
+         * fixed in release 3.1.
+         * https://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.8.12.4.html#sect_C.8.12.4.1.1
+         **/
+        if (tokens.size() < 3 ||
+            tokens[2] == "VOLUME" ||
+            tokens[2] == "THUMBNAIL" /* "may be the apex (lowest resolution) layer of a Multi-Resolution Pyramid" */)
         {
           if (instance->HasBackgroundColor())
           {
@@ -186,6 +192,8 @@ namespace OrthancWSI
 
     for (size_t i = 0; i < instances_.size(); i++)
     {
+      assert(instances_[i] != NULL);
+
       if (i == 0 ||
           instances_[i - 1]->GetTotalWidth() != instances_[i]->GetTotalWidth())
       {
@@ -196,6 +204,8 @@ namespace OrthancWSI
         assert(levels_.back() != NULL);
         levels_.back()->AddInstance(*instances_[i]);
       }
+
+      instances_[i]->SetLevel(levels_.size() - 1);
     }
   }
 
@@ -266,4 +276,35 @@ namespace OrthancWSI
     assert(!instances_.empty() && instances_[0] != NULL);
     return instances_[0]->GetPhotometricInterpretation();
   }
+
+
+  bool DicomPyramid::LookupImagedVolumeSize(double& width,
+                                            double& height) const
+  {
+    bool found = false;
+
+    for (size_t i = 0; i < instances_.size(); i++)
+    {
+      assert(instances_[i] != NULL);
+
+      if (instances_[i]->IsLevel(0) &&  // Only consider the finest level
+          instances_[i]->HasImagedVolumeSize())
+      {
+        if (!found)
+        {
+          found = true;
+          width = instances_[i]->GetImagedVolumeWidth();
+          height = instances_[i]->GetImagedVolumeHeight();
+        }
+        else if (!ImageToolbox::IsNear(width, instances_[i]->GetImagedVolumeWidth()) ||
+                 !ImageToolbox::IsNear(height, instances_[i]->GetImagedVolumeHeight()))
+        {
+          LOG(WARNING) << "Inconsistency of imaged volume width/height in series: " << seriesId_;
+          return false;
+        }
+      }
+    }
+
+    return found;
+  }
 }
