@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -51,11 +51,16 @@ namespace OrthancWSI
     uint8_t                             backgroundRed_;
     uint8_t                             backgroundGreen_;
     uint8_t                             backgroundBlue_;
+    bool                                hasImagedVolumeSize_;
+    double                              imagedVolumeWidth_;
+    double                              imagedVolumeHeight_;
+    bool                                hasLevel_;
+    unsigned int                        level_;
 
     void Load(OrthancStone::IOrthancConnection&  orthanc,
               const std::string& instanceId);
 
-    void Deserialize(const std::string& content);
+    bool Deserialize(const std::string& content);
 
   public:
     DicomPyramidInstance(OrthancStone::IOrthancConnection&  orthanc,
@@ -125,5 +130,18 @@ namespace OrthancWSI
     uint8_t GetBackgroundGreen() const;
 
     uint8_t GetBackgroundBlue() const;
+
+    bool HasImagedVolumeSize() const
+    {
+      return hasImagedVolumeSize_;
+    }
+
+    double GetImagedVolumeWidth() const;
+
+    double GetImagedVolumeHeight() const;
+
+    void SetLevel(unsigned int level);
+
+    bool IsLevel(unsigned int level) const;
   };
 }
