@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -23,6 +24,8 @@
 #include "../PrecompiledHeadersWSI.h"
 #include "DicomPyramidLevel.h"
 
+#include "../ImageToolbox.h"
+
 #include <Logging.h>
 #include <OrthancException.h>
 
@@ -131,15 +134,39 @@ namespace OrthancWSI
       assert(tile.instance_ != NULL);
       DicomPyramidInstance& instance = *tile.instance_;
 
-      std::string uri = ("/instances/" + instance.GetInstanceId() + 
-                         "/frames/" + boost::lexical_cast<std::string>(tile.frame_) + "/raw");
-
-      orthanc.RestApiGet(raw, uri);
-
       compression = instance.GetImageCompression(orthanc);
       format = instance.GetPixelFormat();
 
-      return true;
+      if (compression == ImageCompression_UseOrthancPreview)
+      {
+        // If the WSI viewer plugin has no built-in support for this transfer syntax,
+        // use the decoding primitives offered by the Orthanc core
+        std::string uri = ("/instances/" + instance.GetInstanceId() +
+                           "/frames/" + boost::lexical_cast<std::string>(tile.frame_) + "/preview");
+        orthanc.RestApiGet(raw, uri);
+
+        if (ImageToolbox::HasPngSignature(raw))  // In theory, Orthanc should always generate PNG by default
+        {
+          compression = ImageCompression_Png;
+          return true;
+        }
+        else if (ImageToolbox::HasJpegSignature(raw))
+        {
+          compression = ImageCompression_Jpeg;
+          return true;
+        }
+        else
+        {
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "Cannot decode a preview image generated by the Orthanc core");
+        }
+      }
+      else
+      {
+        std::string uri = ("/instances/" + instance.GetInstanceId() +
+                           "/frames/" + boost::lexical_cast<std::string>(tile.frame_) + "/raw");
+        orthanc.RestApiGet(raw, uri);
+        return true;
+      }
     }
     else
     {
