@@ -151,9 +151,8 @@ namespace OrthancWSI
           // Fix issue 139: The PixelSpacing information changes at each level
           // https://bitbucket.org/sjodogne/orthanc/issues/139/orthancwsidicomizer-pixelspacing
 
-          // In the 2 lines below, remember to switch X/Y when going from physical to pixel coordinates!
-          float spacingX = volume_.GetWidth() / static_cast<float>(level.height_);
-          float spacingY = volume_.GetHeight() / static_cast<float>(level.width_);
+          float spacingX = volume_.GetWidth() / static_cast<float>(level.width_);
+          float spacingY = volume_.GetHeight() / static_cast<float>(level.height_);
 
           std::string spacing = (boost::lexical_cast<std::string>(spacingX) + '\\' +
                                  boost::lexical_cast<std::string>(spacingY));
@@ -185,11 +184,11 @@ namespace OrthancWSI
       }
 
       std::unique_ptr<DcmItem> functionalGroup(CreateFunctionalGroup(writer->GetFramesCount() + 1,
-                                                                   x * GetTileWidth(), 
-                                                                   y * GetTileHeight(), 
-                                                                   writer->GetTotalWidth(),
-                                                                   writer->GetTotalHeight(),
-                                                                   0.0f /* TODO Z-plane */));
+                                                                     x * GetTileWidth(),
+                                                                     y * GetTileHeight(),
+                                                                     writer->GetTotalWidth(),
+                                                                     writer->GetTotalHeight(),
+                                                                     0.0f /* TODO Z-plane */));
 
       writer->AddFrame(tile, functionalGroup.release());
       FlushInternal(*writer, false);
