@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -27,6 +27,7 @@
 #include "DicomPyramidCache.h"
 #include "IIIF.h"
 #include "RawTile.h"
+#include "../Framework/ColorSpaces.h"
 #include "../Framework/Inputs/DecodedTiledPyramid.h"
 #include "../Framework/Inputs/OnTheFlyPyramid.h"
 #include "../Framework/Inputs/DecodedPyramidCache.h"
@@ -128,6 +129,14 @@ void ServePyramid(OrthancPluginRestOutput* output,
               locker.GetPyramid().GetBackgroundBlue());
       answer["BackgroundColor"] = tmp;
     }
+
+    // New in WSI 3.1
+    double imagedVolumeWidth, imagedVolumeHeight;
+    if (locker.GetPyramid().LookupImagedVolumeSize(imagedVolumeWidth, imagedVolumeHeight))
+    {
+      answer["ImagedVolumeWidth"] = imagedVolumeWidth;
+      answer["ImagedVolumeHeight"] = imagedVolumeHeight;
+    }
   }
 
   std::string s = answer.toStyledString();
@@ -353,10 +362,39 @@ OrthancPluginErrorCode OnChangeCallback(OrthancPluginChangeType changeType,
 }
 
 
+void ServeJavaScriptLibraries(OrthancPluginRestOutput* output,
+                              const char* url,
+                              const OrthancPluginHttpRequest* request)
+{
+  OrthancPluginContext* context = OrthancPlugins::GetGlobalContext();
 
-void ServeFile(OrthancPluginRestOutput* output,
-               const char* url,
-               const OrthancPluginHttpRequest* request)
+  if (request->method != OrthancPluginHttpMethod_Get)
+  {
+    OrthancPluginSendMethodNotAllowed(context, output, "GET");
+  }
+  else
+  {
+    const std::string path = "/" + std::string(request->groups[0]);
+    const char* mime = Orthanc::EnumerationToString(Orthanc::SystemToolbox::AutodetectMimeType(path));
+
+    if (path == "/js/ol.js")
+    {
+      // Adding "charset" is mandatory with OpenLayers 10.4.0, check out "zoomOutLabel" in the source code
+      mime = "application/javascript; charset=utf-8";
+    }
+
+    std::string s;
+    Orthanc::EmbeddedResources::GetDirectoryResource(s, Orthanc::EmbeddedResources::JAVASCRIPT_LIBS, path.c_str());
+
+    const char* resource = s.size() ? s.c_str() : NULL;
+    OrthancPluginAnswerBuffer(context, output, resource, s.size(), mime);
+  }
+}
+
+
+void ServeEmbeddedFile(OrthancPluginRestOutput* output,
+                       const char* url,
+                       const OrthancPluginHttpRequest* request)
 {
   Orthanc::EmbeddedResources::FileResourceId resource;
 
@@ -365,24 +403,24 @@ void ServeFile(OrthancPluginRestOutput* output,
 
   if (f == "viewer.html")
   {
+#if ORTHANC_STANDALONE == 0
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+#else
     resource = Orthanc::EmbeddedResources::VIEWER_HTML;
+#endif
+
     mime = "text/html";
   }
   else if (f == "viewer.js")
   {
+#if ORTHANC_STANDALONE == 0
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+#else
     resource = Orthanc::EmbeddedResources::VIEWER_JS;
+#endif
+
     mime = "application/javascript";
   }
-  else if (f == "ol.js")
-  {
-    resource = Orthanc::EmbeddedResources::OPENLAYERS_JS;
-    mime = "application/javascript";
-  }
-  else if (f == "ol.css")
-  {
-    resource = Orthanc::EmbeddedResources::OPENLAYERS_CSS;
-    mime = "text/css";
-  }
   else if (f == "mirador.html")
   {
     resource = Orthanc::EmbeddedResources::MIRADOR_HTML;
@@ -405,6 +443,33 @@ void ServeFile(OrthancPluginRestOutput* output,
 }
 
 
+#if ORTHANC_STANDALONE == 0
+void ServeSourceFile(OrthancPluginRestOutput* output,
+                     const char* url,
+                     const OrthancPluginHttpRequest* request)
+{
+  // This method should only be used during the development to speed up compilation
+
+  std::string filename(request->groups[0]);
+
+  if (filename != "viewer.html" &&
+      filename != "viewer.js")
+  {
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
+  }
+
+  const boost::filesystem::path path = Orthanc::SystemToolbox::InterpretRelativePath(
+    Orthanc::SystemToolbox::PathFromUtf8(PLUGIN_SOURCE_DIR), filename);
+  const char* mime = Orthanc::EnumerationToString(Orthanc::SystemToolbox::AutodetectMimeType(filename));
+
+  std::string content;
+  Orthanc::SystemToolbox::ReadFile(content, path);
+
+  OrthancPluginAnswerBuffer(OrthancPlugins::GetGlobalContext(), output, content.c_str(), content.size(), mime);
+}
+#endif
+
+
 extern "C"
 {
   ORTHANC_PLUGINS_API int32_t OrthancPluginInitialize(OrthancPluginContext* context)
@@ -439,6 +504,49 @@ extern "C"
     Orthanc::Logging::Initialize(context);
 #endif
 
+    try
+    {
+      /**
+
+         C.10.7.1.1 Encoding of CIELab Values
+
+         Attributes such as Graphic Layer Recommended Display CIELab
+         Value (0070,0401) consist of three unsigned short values:
+
+         An L value linearly scaled to 16 bits, such that 0x0000
+         corresponds to an L of 0.0, and 0xFFFF corresponds to an L of
+         100.0.
+
+         An a* then a b* value, each linearly scaled to 16 bits and
+         offset to an unsigned range, such that 0x0000 corresponds to
+         an a* or b* of -128.0, 0x8080 corresponds to an a* or b* of
+         0.0 and 0xFFFF corresponds to an a* or b* of 127.0
+
+       **/
+
+      OrthancWSI::LABColor lab;
+      if (!OrthancWSI::LABColor::DecodeDicomRecommendedAbsentPixelCIELab(lab, "65535\\0\\0") ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetL(), 100.0, 0.001) ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetA(), -128.0, 0.001) ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetB(), -128.0, 0.001))
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+
+      if (!OrthancWSI::LABColor::DecodeDicomRecommendedAbsentPixelCIELab(lab, "0\\32896\\65535") ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetL(), 0.0, 0.001) ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetA(), 0.0, 0.001) ||
+          !OrthancWSI::ImageToolbox::IsNear(lab.GetB(), 127.0, 0.001))
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+    }
+    catch (Orthanc::OrthancException& e)
+    {
+      LOG(ERROR) << "Exception in startup tests: " << e.What();
+      return -1;
+    }
+
     // Limit the number of PNG transcoders to the number of available
     // hardware threads (e.g. number of CPUs or cores or
     // hyperthreading units)
@@ -449,7 +557,8 @@ extern "C"
 
     OrthancPlugins::SetDescription(ORTHANC_PLUGIN_NAME, "Provides a Web viewer of whole-slide microscopic images within Orthanc.");
 
-    OrthancWSI::DicomPyramidCache::InitializeInstance(10 /* Number of pyramids to be cached - TODO parameter */);
+    OrthancWSI::DicomPyramidCache::InitializeInstance(10 /* Number of pyramids to be cached - TODO parameter */,
+                                                      true /* Use the metadata cache - Should be "false" only during development */);
 
     {
       std::unique_ptr<OrthancWSI::OrthancPyramidFrameFetcher> fetcher(
@@ -465,10 +574,16 @@ extern "C"
 
     OrthancPluginRegisterOnChangeCallback(OrthancPlugins::GetGlobalContext(), OnChangeCallback);
 
-    OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(ol.css)", true);
-    OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(ol.js)", true);
-    OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(viewer.html)", true);
-    OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(viewer.js)", true);
+    OrthancPlugins::RegisterRestCallback<ServeJavaScriptLibraries>("/wsi/libs/(.*)", true);
+
+#if ORTHANC_STANDALONE == 1
+    OrthancPlugins::RegisterRestCallback<ServeEmbeddedFile>("/wsi/app/(viewer.html)", true);
+    OrthancPlugins::RegisterRestCallback<ServeEmbeddedFile>("/wsi/app/(viewer.js)", true);
+#else
+    OrthancPlugins::RegisterRestCallback<ServeSourceFile>("/wsi/app/(viewer.html)", true);
+    OrthancPlugins::RegisterRestCallback<ServeSourceFile>("/wsi/app/(viewer.js)", true);
+#endif
+
     OrthancPlugins::RegisterRestCallback<ServePyramid>("/wsi/pyramids/([0-9a-f-]+)", true);
     OrthancPlugins::RegisterRestCallback<ServeTile>("/wsi/tiles/([0-9a-f-]+)/([0-9-]+)/([0-9-]+)/([0-9-]+)", true);
     OrthancPlugins::RegisterRestCallback<ServeFramePyramid>("/wsi/frames-pyramids/([0-9a-f-]+)/([0-9-]+)", true);
@@ -527,12 +642,12 @@ extern "C"
 
     if (serveMirador)
     {
-      OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(mirador.html)", true);
+      OrthancPlugins::RegisterRestCallback<ServeEmbeddedFile>("/wsi/app/(mirador.html)", true);
     }
 
     if (serveOpenSeadragon)
     {
-      OrthancPlugins::RegisterRestCallback<ServeFile>("/wsi/app/(openseadragon.html)", true);
+      OrthancPlugins::RegisterRestCallback<ServeEmbeddedFile>("/wsi/app/(openseadragon.html)", true);
     }
 
     {
