@@ -21,31 +21,10 @@
  **/
 
 
-// For IE compatibility
-if (!window.console) window.console = {};
-if (!window.console.log) window.console.log = function () { };
-
-
-// http://stackoverflow.com/a/21903119/881731
-function GetUrlParameter(sParam) 
+function IsNear(a, b)
 {
-  var sPageURL = decodeURIComponent(window.location.search.substring(1));
-  var sURLVariables = sPageURL.split('&');
-  var sParameterName;
-  var i;
-
-  for (i = 0; i < sURLVariables.length; i++) 
-  {
-    sParameterName = sURLVariables[i].split('=');
-
-    if (sParameterName[0] === sParam) 
-    {
-      return sParameterName[1] === undefined ? '' : sParameterName[1];
-    }
-  }
-
-  return '';
-};
+  return Math.abs(a - b) <= 0.01;
+}
 
 
 function InitializePyramid(pyramid, tilesBaseUrl)
@@ -61,8 +40,21 @@ function InitializePyramid(pyramid, tilesBaseUrl)
   var imagedVolumeHeight = pyramid['ImagedVolumeHeight'];
   if (imagedVolumeWidth !== undefined &&
       imagedVolumeHeight !== undefined) {
-    metersPerUnit = parseFloat(imagedVolumeWidth) / (1000.0 * parseFloat(height));
-    //metersPerUnit = parseFloat(imagedVolumeHeight) / (1000.0 * parseFloat(width));
+    var metersPerUnitX = parseFloat(imagedVolumeWidth) / (1000.0 * parseFloat(width));
+    var metersPerUnitY = parseFloat(imagedVolumeHeight) / (1000.0 * parseFloat(height));
+    if (IsNear(metersPerUnitX / metersPerUnitY, 1)) {
+      metersPerUnit = metersPerUnitX;
+    } else {
+      // Backward compatibility with OrthancWSIDicomizer <= 3.2, where X/Y were swapped
+      metersPerUnitX = parseFloat(imagedVolumeWidth) / (1000.0 * parseFloat(height));
+      metersPerUnitY = parseFloat(imagedVolumeHeight) / (1000.0 * parseFloat(width));
+      if (IsNear(metersPerUnitX / metersPerUnitY, 1)) {
+        metersPerUnit = metersPerUnitX;
+      } else {
+        console.error('Anisotropic pixel spacing (may result from an inconsistency ' +
+                      'in the imaged volume size), not showing the scale');
+      }
+    }
   }
 
   // Maps always need a projection, but Zoomify layers are not geo-referenced, and
@@ -77,6 +69,19 @@ function InitializePyramid(pyramid, tilesBaseUrl)
 
   var extent = [0, -height, width, 0];
 
+  var rotateControl = new ol.control.Rotate({
+    autoHide: false,  // Show the button even if rotation is 0
+    resetNorth: function() {  // Disable the default action
+    }
+  });
+
+  new bootstrap.Popover(rotateControl.element, {
+    placement: 'right',
+    container: 'body',
+    html: true,
+    content: $('#popover-content')
+  });
+
   // Disable the rotation of the map, and inertia while panning
   // http://stackoverflow.com/a/25682186
   var interactions = ol.interaction.defaults.defaults({
@@ -93,11 +98,22 @@ function InitializePyramid(pyramid, tilesBaseUrl)
   var controls = ol.control.defaults.defaults({
     attribution: false
   }).extend([
+    rotateControl,
     new ol.control.ScaleLine({
       minWidth: 100
     })
   ]);
 
+  const params = new URLSearchParams(document.location.search);
+  if (params.has('description')) {
+    controls.extend([
+      new ol.control.Attribution({
+        attributions: params.get('description'),
+        collapsible: false
+      })
+    ]);
+  }
+
 
   var layer = new ol.layer.Tile({
     extent: extent,
@@ -131,15 +147,39 @@ function InitializePyramid(pyramid, tilesBaseUrl)
   });
 
   map.getView().fit(extent, map.getSize());
+
+
+  $('#rotation-slider').on('input change', function() {
+    map.getView().setRotation(this.value / 180 * Math.PI);
+  });
+
+  $('#rotation-reset').click(function() {
+    $('#rotation-slider').val(0).change();
+  });
+
+  $('#rotation-minus90').click(function() {
+    var angle = parseInt($('#rotation-slider').val()) - 90;
+    if (angle < -180) {
+      angle += 360;
+    }
+    $('#rotation-slider').val(angle).change();
+  });
+
+  $('#rotation-plus90').click(function() {
+    var angle = parseInt($('#rotation-slider').val()) + 90;
+    if (angle > 180) {
+      angle -= 360;
+    }
+    $('#rotation-slider').val(angle).change();
+  });
 }
 
 
 $(document).ready(function() {
-  var seriesId = GetUrlParameter('series');
-  var instanceId = GetUrlParameter('instance');
+  const params = new URLSearchParams(document.location.search);
 
-  if (seriesId.length != 0)
-  {
+  if (params.has('series')) {
+    var seriesId = params.get('series');
     $.ajax({
       url : '../pyramids/' + seriesId,
       error: function() {
@@ -149,15 +189,13 @@ $(document).ready(function() {
         InitializePyramid(pyramid, '../tiles/' + seriesId + '/');
       }
     });
-  }
-  else if (instanceId.length != 0)
-  {
-    var frameNumber = GetUrlParameter('frame');
-    if (frameNumber.length == 0)
-    {
-      frameNumber = 0;
+  } else if (params.has('instance')) {
+    var frameNumber = 0;
+    if (params.has('frame')) {
+      frameNumber = params.get('frame');
     }
 
+    var instanceId = params.get('instance');
     $.ajax({
       url : '../frames-pyramids/' + instanceId + '/' + frameNumber,
       error: function() {
@@ -167,9 +205,7 @@ $(document).ready(function() {
         InitializePyramid(pyramid, '../frames-tiles/' + instanceId + '/' + frameNumber + '/');
       }
     });
-  }
-  else
-  {
+  } else {
     alert('Error - No series ID and no instance ID specified!');
   }
 });
